package cv08.server;

import cv08.server.action.*;

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.Socket;
import java.util.*;

/**
 * Klient pripojeny k serveru
 *
 * @author Karel Cemus
 */
public class Client implements Runnable {

    /** network socket fo communication to the client */
    private final Socket socket;

    /** stream pro prijimani zprav od klienta */
    private final Scanner input;

    /** stream pro posilani zprav klientovi */
    private final DataOutputStream output;

    /** uzivatelova prezdivka */
    private String nickname;

    /** mistnosti, ve kterych klient je */
    private final Set<Room> rooms = new HashSet<Room>();

    public Client( Socket socket ) throws IOException {

        // socket pres ktery komunikujeme s uzivatelem
        this.socket = socket;

        // streamy pro posilani a cteni dat
        input = new Scanner( socket.getInputStream() );
        output = new DataOutputStream( socket.getOutputStream() );
    }

    /** odeslat klientovi zprávu */
    public boolean send( String message ) {
        try {
            output.writeBytes( message );
            output.writeByte( '\n' );
            return true;
        } catch ( IOException e ) {
            return false;
        }
    }

    /** uzivatel vstoupil do mistnosti */
    public void enterRoom( Room room ) {
        rooms.add( room );
        send( "STATUS 02 Entered room " + room.getName() );
    }

    /** uzivatel opustil do mistnosti */
    public void leaveRoom( Room room ) {
        rooms.remove( room );
        send( "STATUS 03 Left room " + room.getName() );
    }

    @Override
    public void run() {

        // nacist nick uzivatele
        if ( "NICK".equals( input.next() ) ) {
            nickname = input.next();
            send( "STATUS 01 Nick accepted." );
        } else {
            send( "ERR 05 Expected commnad NICK. Client is being disconnected ..." );
            close();
            return;
        }

        System.out.println( String.format( "User '%s' is connected.", nickname ) );

        try {

            // dokud chodi prikazy, tak je budeme cist
            while ( input.hasNext() ) {

                // server se vypina, skoncime
                if ( Thread.currentThread().isInterrupted() ) break;

                // nacti dalsi prikaz
                String command = input.next();

                // podle prikazu se zachovame
                switch ( command ) {
                    // vypsat seznam mistnosti
                    case "LIST":
                        // ziskani mistnosti a zapsani do jedne radky oddelene mezerou
                        Worker.addAction( new ListAction( this ) );
                        break;
                    case "CREATE":
                        String newRoom = input.next();
                        // vytvoreni mistnosti
                        Worker.addAction( new CreateRoomAction( this, newRoom ) );
                        break;
                    // registrace do mistnosti
                    case "ENTER":
                        String enter = input.next();
                        Worker.addAction( new EnterRoomAction( this, enter ) );
                        break;
                    // opusteni mistnosti
                    case "LEAVE":
                        String leave = input.next();
                        Worker.addAction( new LeaveRoomAction( this, leave ) );
                        break;
                    // zadost o odeslani zpravy
                    case "SEND":
                        String room = input.next();
                        String text = input.nextLine().trim();
                        Worker.addAction( new SendAction( this, room, text ) );
                        break;
                    default:
                        // akce neni rozpoznana, preskoc zbytek radku
                        Worker.addAction( new UnknownAction( this ) );
                }
            }

        } finally {
            // nakonec ho vyhodime ze vsech mistnosti
            for ( Room room : rooms )
                Worker.addAction( new LeaveRoomAction( this, room.getName() ) );

            close();
            System.out.println( String.format( "User '%s' was disconnected.", nickname ) );
        }
    }

    public String getNickname() {
        return nickname;
    }

    private void close() {
        try {
            socket.close();
        } catch ( IOException ignored ) {}
    }
}
